
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.text.DecimalFormat;

/**
 *
 * @author nicolas
 */
public class Fart {
    private double alpha, beta, rho;            // Configuracion del algoritmo de FART
    private int activeNeurone, nearestNeuron;   // Numero de neuronas activas y neurona mas cercana
    private double[][] w= new double[1000][4];
    private int verbose = 0;

    // Inicia los datos numéricos con valores predefinidos.
    public Fart() {
        this.alpha = 0.000001;
        this.beta = 1;
        this.rho = 0.95;
        this.restart();
    }   // End of public Fart() {

    // Inicia los datos numéricos con valores predefinidos.
    public Fart(double alpha, double beta, double rho) {
        this.alpha = alpha;
        this.beta = beta;
        this.rho = rho;
        this.restart();
    }   // End of public Fart(double alpha, double beta, double rho) {

    // Setea el parámetro Alpha
    public void setAlpha(double alpha) {    this.alpha = alpha;     }

    // Setea el  parámetro Beta
    public void setBeta(double beta) {      this.beta = beta;       }

    // Setea el parametro Rho
    public void setRho(double rho) {        this.rho = rho;         }

    // Retorna el parámetro Alpha
    public double getAlpha() {          return alpha;       }    

    // Retorna el parámetro Beta
    public double getBeta() {           return beta;        }

    // Retorna el parámetro Rho
    public double getRho() {            return rho;         }

    // Setea el nivel de verbose, por defecto es el mas bajo
    public void setVerbose(int verboseLevel) {     verbose = verboseLevel;  }

    // Retorna el nivel de verbose
    public int getVerbose() {   return verbose;  }

    // Actualiza el arreglo de pesos w[neurona][peso]
    public void setW(double[][] w) {    this.w = w;     }
    
    // Setea el numero de neuronas que están activas (comprometidas).
    public void setActiveNeurone(int actives) { this.activeNeurone = actives;   }
    
    public void setNearestNeuron(int NS) {  this.nearestNeuron = NS;    }

    // Retorna la última neurona que fue seleccionada.
    public int getNearestNeuron(){  return this.nearestNeuron;  }

    // Reinicia el algoritmo: Reinicia pesos=1 y neuronas activas=0.
    public void restart() {
        this.activeNeurone = 0;
        this.nearestNeuron = 0;

        for (int i = 0; i < 4; i++)
            for(int j = 0; j < 1000; j++) {
                this.w[j][i] = 1;
            }
    }   // End of public void restart() {

    // Computo del algoritmo de FART para coordenadas normalizadas x e y
    public void calculation(Double xcoord, Double ycoord) {
        int e, i, j, oldOrNew;
        double[] T = new double[1000], S = new double[1000];
        double[] min = new double[4];
        double[] input = new double[4];
        double sum1, sum2;
        int escogido = 0, maxj = 0;
        
        // Se define la entrada FART en la forma:
        // (input[1], input[2], input[3], input[4]) = (x, y, x^c, y^c)
        // a partir de las entradas (X, Y) en coordenadas de panel.
        input[0] = xcoord;
        input[1] = ycoord;
        input[2] = 1-input[0];
        input[3] = 1-input[1];
        e = 0;

        // Se verifica si alguna neurona activa (si es que hay) resuena con la entrada.
        if (activeNeurone > 0) {
            for (j = 0; j < activeNeurone; j++) {
                for (i = 0; i < 4; i++){
                    if(w[j][i] < input[i])
                        min[i] = w[j][i];
                    else
                        min[i] = input[i];
                }
                sum1 = min[0] + min[1] + min[2] + min[3];
                sum2 = w[j][0] + w[j][1] + w[j][2] + w[j][3];
                T[j] = sum1/(alpha + sum2);
            }

            int c = 0;
            // Se busca la neurona que más resuene y se verifica si cumple con la condición de vigilancia.
            // Esto hasta que se encuentre una que cumpla o se hayan revisado todas las neuronas activas.
            while ( (e == 0) && ((activeNeurone - c) > 0) ) {
                for ( j = 1; j < activeNeurone; j++)
                    if (T[j] > T[maxj])
                        maxj = j;
                
                for (i = 0; i < 4; i++) {
                    if (w[maxj][i] < input[i])
                        min[i] = w[maxj][i];
                    else
                        min[i] = input[i];
                }
                sum1 = min[0] + min[1] + min[2] + min[3];
                sum2 = input[0] + input[1] + input[2] + input[3];
                S[maxj] = sum1/sum2;

                if (S[maxj] >= rho)
                    e = 1;      // Se encontró una neurona ya activa que resonó y cumple con el vigilante.
                else {
                    T[maxj] =- 1;
                    c++;
                }
            }
        }

        // Si se encontró (e == 1) o no (e == 0)
        if ( e == 1 ) {
            escogido = maxj;        // escogido=nro o id. de la neurona escogida  
            oldOrNew = 1;           // La neurona escogida es vieja (activada previamente)
        }
        else {
            escogido = activeNeurone;       // escogido=nro o id. de la primera neurona no activa
            // Nota:neuronas escogidas van de 0 a [activas-1].
            setActiveNeurone(activeNeurone + 1);    // Indica que hay una neurona activa más.
            oldOrNew = 2;                   // Se escogió una nueva neurona.
        }

        // Se actualizan los pesos de la neurona escogida
        for (i = 0; i < 4; i++) {
            if ( w[escogido][i] < input[i] )
                min[i] = w[maxj][i];
            else
                min[i] = input[i];
                w[escogido][i] = beta * min[i];
        }

        // Se actualiza el arreglo de pesos w
        setW(w);

        // Se guarda el número de la neurona, que fue escogida.
        setNearestNeuron(escogido);       
    }   // End of public void calculation(Double xcoord, Double ycoord) {

    // Imprime los pesos del mapa
    public void printMap() {
        int i;

        // Impresión en consola de las categorias con el nivel de verbose mas bajo
        if (verbose == 0) {
            if (activeNeurone > 0)
                for(i = 0; i < activeNeurone; i++) {
                    System.out.print(((w[i][0])) + " " + ((w[i][1])) + " ");
                    System.out.println((1-w[i][0]-w[i][2]) + " " + (1-w[i][1]-w[i][3]));
                }
        }

        if (verbose == 1) {
            if (activeNeurone > 0)
                for(i = 0; i < activeNeurone; i++) {
                    System.out.println("Categoria " + (i+1)); 
                    System.out.print("\t");
                    System.out.print("X = "+((w[i][0])) + " Y = "+((w[i][1])));
                    System.out.println(" W = "+(1-w[i][0]-w[i][2]) + " H = "+(1-w[i][1]-w[i][3]));
                    System.out.println();
                }
            System.out.println("---------------------------------------------------------------------");
            System.out.println();
        }

        FileOutputStream os;
        DecimalFormat df = new DecimalFormat("000.00"); 

        try {
            os = new FileOutputStream("fart.log");
            PrintStream ps = new PrintStream(os);

            ps.println("X\tY\tW\tH");

            if (activeNeurone > 0)
                for(i = 0; i < activeNeurone; i++) {                
                    ps.print(df.format(w[i][0]) +"\t"+ df.format(w[i][1]));
                    ps.println("\t"+ df.format(1-w[i][0]-w[i][2]) +"\t"+ df.format(1-w[i][1]-w[i][3]));
                }
            ps.close();
        } catch (FileNotFoundException ex) {    ex.printStackTrace();   }
    }   // End of public void printMap() {
}   // End of public class Fart {
